@@ -1,8 +1,21 @@
 /* deflate.c -- compress data using the deflation algorithm
- * Copyright (C) 1992-1993 Jean-loup Gailly
- * This is free software; you can redistribute it and/or modify it under the
- * terms of the GNU General Public License, see the file COPYING.
- */
+
+   Copyright (C) 1999, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1992-1993 Jean-loup Gailly
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /*
  *  PURPOSE
@@ -55,12 +68,13 @@
  *      void lm_init (int pack_level, ush *flags)
  *          Initialize the "longest match" routines for a new file
  *
- *      ulg deflate (void)
+ *      off_t deflate (void)
  *          Processes a new input file and return its compressed length. Sets
  *          the compressed length, crc, deflate flags and internal file
  *          attributes.
  */
 
+#include <config.h>
 #include <stdio.h>
 
 #include "tailor.h"
@@ -68,7 +82,7 @@
 #include "lzw.h" /* just for consistency checking */
 
 #ifdef RCSID
-static char rcsid[] = "$Id: deflate.c,v 0.15 1993/06/24 10:53:53 jloup Exp $";
+static char rcsid[] = "$Id: deflate.c,v 1.4 2006/11/20 08:40:33 eggert Exp $";
 #endif
 
 /* ===========================================================================
@@ -248,7 +262,7 @@ local config configuration_table[10] = {
  *  Prototypes for local functions.
  */
 local void fill_window   OF((void));
-local ulg deflate_fast   OF((void));
+local off_t deflate_fast OF((void));
 
       int  longest_match OF((IPos cur_match));
 #ifdef ASMV
@@ -289,7 +303,7 @@ void lm_init (pack_level, flags)
 {
     register unsigned j;
 
-    if (pack_level < 1 || pack_level > 9) error("bad pack level");
+    if (pack_level < 1 || pack_level > 9) gzip_error ("bad pack level");
     compr_level = pack_level;
 
     /* Initialize the hash table. */
@@ -494,7 +508,7 @@ local void check_match(start, match, length)
         fprintf(stderr,
             " start %d, match %d, length %d\n",
             start, match, length);
-        error("invalid match");
+        gzip_error ("invalid match");
     }
     if (verbose > 1) {
         fprintf(stderr,"\\[%d,%d]", start-match, length);
@@ -577,7 +591,7 @@ local void fill_window()
  * new strings in the dictionary only for unmatched strings or for short
  * matches. It is used only for the fast compression options.
  */
-local ulg deflate_fast()
+local off_t deflate_fast()
 {
     IPos hash_head; /* head of the hash chain */
     int flush;      /* set if current block must be flushed */
@@ -623,7 +637,7 @@ local ulg deflate_fast()
                      * the next lookahead bytes will be emitted as literals.
                      */
                 } while (--match_length != 0);
-	        strstart++; 
+	        strstart++;
             } else {
 	        strstart += match_length;
 	        match_length = 0;
@@ -638,7 +652,7 @@ local ulg deflate_fast()
             Tracevv((stderr,"%c",window[strstart]));
             flush = ct_tally (0, window[strstart]);
             lookahead--;
-	    strstart++; 
+	    strstart++;
         }
         if (flush) FLUSH_BLOCK(0), block_start = strstart;
 
@@ -658,16 +672,13 @@ local ulg deflate_fast()
  * evaluation for matches: a match is finally adopted only if there is
  * no better match at the next window position.
  */
-ulg deflate()
+off_t deflate()
 {
     IPos hash_head;          /* head of hash chain */
     IPos prev_match;         /* previous match */
     int flush;               /* set if current block must be flushed */
     int match_available = 0; /* set if previous match exists */
     register unsigned match_length = MIN_MATCH-1; /* length of best match */
-#ifdef DEBUG
-    extern long isize;        /* byte length of input file, for debug only */
-#endif
 
     if (compr_level <= 3) return deflate_fast(); /* optimized for speed */
 
@@ -748,7 +759,7 @@ ulg deflate()
             strstart++;
             lookahead--;
         }
-        Assert (strstart <= isize && lookahead <= isize, "a bit too far");
+        Assert (strstart <= bytes_in && lookahead <= bytes_in, "a bit too far");
 
         /* Make sure that we always have enough lookahead, except
          * at the end of the input file. We need MAX_MATCH bytes
